 /*
 /********************************************************************/
 /*                                                                  */
 /* Program name: ExecOrder                                          */
 /*                                                                  */
 /* Description: Sample C program that puts order messages to        */
 /*              a message queue                                     */
 /*                                                                  */
 /*  Statement:     Licensed Materials - Property of IBM             */
 /*                                                                  */
 /*                 MQSeries SupportPac WA02                         */
 /*                 (C) Copyright IBM Corp. 2000                     */
 /*                                                                  */
 /********************************************************************/
 /*                                                                  */
 /* Function:                                                        */
 /*                                                                  */
 /*                                                                  */
 /*   ExecOrder is a sample C program to put order messages on a     */
 /*   message queue. It's used as sample program which puts a        */
 /*   message as C-Structure on a queue which will be formattet by   */
 /*   MQSeries Integrator in to a MQSeries Workflow Execute Process  */
 /*   XML Message.                                                   */
 /*                                                                  */
 /*                                                                  */
 /*    Program logic:                                                */
 /*         MQOPEN target queue for OUTPUT                           */
 /*         .  MQPUT C-Structure                                     */
 /*         MQCLOSE target queue                                     */
 /*                                                                  */
 /*                                                                  */
 /********************************************************************/
 /*                                                                  */
 /*  ExecOrder has 3 parameters                                      */
 /*                 - the name of the target queue (required)        */
 /*                 - queue manager name (optional)                  */
 /*                 - customer name (optional for ExecOrder          */
 /*                     but used in Workflow for accept Order path)  */
 /*                                                                  */
 /********************************************************************/
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
    /* includes for MQI */
 #include <cmqc.h>
    /* order message structure */
 #include <fmct3act.h>

 int main(int argc, char **argv)
 {

   /*   Declare MQI structures needed                                */
   MQOD     od = {MQOD_DEFAULT};    /* Object Descriptor             */
   MQMD     md = {MQMD_DEFAULT};    /* Message Descriptor            */
   MQPMO   pmo = {MQPMO_DEFAULT};   /* put message options           */
   MQGMO   gmo = {MQGMO_DEFAULT};   /* get message options           */

   MQHCONN  Hcon;                   /* connection handle             */
   MQHOBJ   Hobj;                   /* object handle                 */
   MQLONG   O_options;              /* MQOPEN options                */
   MQLONG   C_options;              /* MQCLOSE options               */
   MQLONG   CompCode;               /* completion code               */
   MQLONG   OpenCode;               /* MQOPEN completion code        */
   MQLONG   Reason;                 /* reason code                   */
   MQLONG   CReason;                /* reason code for MQCONN        */

   MQLONG   messlen = 0;            /* message length                */
   char     buffer[2048];           /* message buffer                */
   char     QMName[MQ_Q_MGR_NAME_LENGTH] = "MQSIQM";
                                    /* default queue manager name    */
   char     ReplyQName[MQ_Q_NAME_LENGTH] = "ExecOrderIn";
                                    /* reply to queue name           */

   Order_StartOrder sOrder;
   Order_Message    *sOrder_Reply;


   printf("############################################\n");
   printf("### ExecOrder: Start MQSWF Order Process ###\n");
   printf("############################################\n");
   if (argc < 3)
   {
     printf("Required parameter missing - queue name\n");
     printf("Invoke ExecOrder <queue name> <qmgr name> <CustomerLastName>\n");
     printf("\nSample: ExecOrder ExecOrderOut MQSIQM Smith\n");
     exit(99);
   }

   /******************************************************************/
   /*                                                                */
   /*   Connect to queue manager                                     */
   /*                                                                */
   /******************************************************************/
   if (argc > 2)
     strcpy(QMName, argv[2]);

   MQCONN(QMName,                   /* queue manager                 */
          &Hcon,                    /* connection handle             */
          &CompCode,                /* completion code               */
          &CReason);                /* reason code                   */

   /* report reason and stop if it failed     */
   if (CompCode == MQCC_FAILED)
   {
     printf("Error: MQCONN ended with reason code %ld\n", CReason);
     exit( (int)CReason );
   }

   /******************************************************************/
   /*                                                                */
   /*   Use parameter as the name of the target queue                */
   /*                                                                */
   /******************************************************************/
   strncpy(od.ObjectName, argv[1], (size_t)MQ_Q_NAME_LENGTH);

   /******************************************************************/
   /*                                                                */
   /*   Open the target message queue for output                     */
   /*                                                                */
   /******************************************************************/
   O_options = MQOO_OUTPUT          /* open queue for output         */
           + MQOO_FAIL_IF_QUIESCING;/* but not if MQM stopping       */
   MQOPEN(Hcon,                     /* connection handle             */
          &od,                      /* object descriptor for queue   */
          O_options,                /* open options                  */
          &Hobj,                    /* object handle                 */
          &OpenCode,                /* MQOPEN completion code        */
          &Reason);                 /* reason code                   */

   /* report reason, if any; stop if failed      */
   if (Reason != MQRC_NONE)
   {
     printf("Error: MQOPEN ended with reason code %ld\n", Reason);
   }

   if (OpenCode == MQCC_FAILED)
   {
     printf("Error: unable to open queue for output\n");
   }

   /******************************************************************/
   /*                                                                */
   /*   Read lines from the file and put them to the message queue   */
   /*   Loop until null line or end of file, or there is a failure   */
   /*                                                                */
   /******************************************************************/
   CompCode = OpenCode;             /* use MQOPEN result             */
   if(CompCode != MQCC_FAILED)
   {

     memset(buffer, '\0', sizeof(buffer));

     memcpy(md.Format,              /* character string format       */
            MQFMT_STRING, (size_t)MQ_FORMAT_LENGTH);

     md.MsgType = MQMT_REQUEST;     /* set message type to request   */

     strncpy(md.ReplyToQ,           /* set reply to queue name       */
            ReplyQName, (size_t)MQ_Q_NAME_LENGTH);
     strncpy(md.ReplyToQMgr,        /* set reply to queue mgr name   */
            QMName, (size_t)MQ_Q_MGR_NAME_LENGTH);

     pmo.Options |= MQPMO_NEW_MSG_ID;
     pmo.Options |= MQPMO_NEW_CORREL_ID;

     messlen = sizeof(sOrder);      /* get total message length      */
     memset((void *)&sOrder, '\0', messlen);

     // the following part sets the C structure, partly hardcoded.

     printf(" Set C Structure:\n");
     if (argc > 3)
     {
       strncpy(sOrder.PersonInfo.CustomerLastName, argv[3],
                sizeof(sOrder.PersonInfo.CustomerLastName) );
     }
     printf("  Order_StartOrder.PersonInfo.CustomerLastName: %s\n",
                            sOrder.PersonInfo.CustomerLastName);
     sOrder.PersonInfo.CustomerNumber = 123;
     printf("  Order_StartOrder.PersonInfo.CustomerNumber: %d\n",
                                   sOrder.PersonInfo.CustomerNumber);
     sOrder.PartsInfo.PartNumber = 600;
     printf("  Order_StartOrder.PartsInfo.PartNumber: %d\n",
                                    sOrder.PartsInfo.PartNumber);
     sOrder.PartsInfo.PartOrderQty = 2;
     printf("  Order_StartOrder.PartsInfo.PartOrderQty: %d\n",
                                sOrder.PartsInfo.PartOrderQty);
     strcpy(sOrder.PartsInfo.PartName, "Thinkpad");
     printf("  Order_StartOrder.PartsInfo.PartName: %s\n",
                                    sOrder.PartsInfo.PartName);
     memcpy(buffer, (void *)&sOrder, sizeof(sOrder));


     /****************************************************************/
     /*                                                              */
     /*   Put buffer to the message queue                            */
     /*                                                              */
     /****************************************************************/

     MQPUT(Hcon,                /* connection handle                 */
           Hobj,                /* object handle                     */
           &md,                 /* message descriptor                */
           &pmo,                /* default options (datagram)        */
           messlen,             /* message length                    */
           buffer,              /* message buffer                    */
           &CompCode,           /* completion code                   */
           &Reason);            /* reason code                       */

     /* report reason, if any */
     if (Reason != MQRC_NONE)
     {
       printf("Error: MQPUT ended with reason code %ld\n", Reason);
     }
     else
     {
       printf("=== MQPUT ended with reason code %ld ===\n", Reason);
     }
   }

   /******************************************************************/
   /*                                                                */
   /*   Close the target queue (if it was opened)                    */
   /*                                                                */
   /******************************************************************/
   if (OpenCode != MQCC_FAILED)
   {
     C_options = 0;                  /* no close options             */
     MQCLOSE(Hcon,                   /* connection handle            */
             &Hobj,                  /* object handle                */
             C_options,
             &CompCode,              /* completion code              */
             &Reason);               /* reason code                  */

     /* report reason, if any     */
     if (Reason != MQRC_NONE)
     {
       printf("Error: MQCLOSE ended with reason code %ld\n", Reason);
     }
   }

   /******************************************************************/
   /*                                                                */
   /*   Open the reply message queue to get reply                    */
   /*                                                                */
   /******************************************************************/

   strncpy(od.ObjectName,           /* set reply to queue name       */
            ReplyQName, (size_t)MQ_Q_NAME_LENGTH);

   O_options = MQOO_INPUT_SHARED     /* open queue for shared        */
           + MQOO_FAIL_IF_QUIESCING; /* but not if MQM stopping      */
   MQOPEN(Hcon,                      /* connection handle            */
          &od,                       /* object descriptor for queue  */
          O_options,                 /* open options                 */
          &Hobj,                     /* object handle                */
          &OpenCode,                 /* MQOPEN completion code       */
          &Reason);                  /* reason code                  */

   /* report reason, if any; stop if failed      */
   if (Reason != MQRC_NONE)
   {
     printf("MQOPEN ended with reason code %ld\n", Reason);
     return( Reason);
   }

   if (OpenCode == MQCC_FAILED)
   {
     printf("unable to open queue for output\n");
     return( Reason);
   }
   if(CompCode != MQCC_FAILED)
   {

     printf("\nNow wait for reply message in queue %s ....\n",
                                                    od.ObjectName );
     messlen = sizeof(buffer);
     gmo.Options= MQGMO_WAIT;
     gmo.WaitInterval= MQWI_UNLIMITED;
                                     /* wait unlimited until reply   */
                                     /* arrives                      */
     gmo.MatchOptions = MQMO_MATCH_CORREL_ID;

     memcpy((char*)md.CorrelId,      /* set CorrelId                 */
             (char *)md.MsgId, (size_t)MQ_Q_NAME_LENGTH);

     memcpy(md.MsgId,           /* reset MsgId                       */
              MQMI_NONE, sizeof(md.MsgId) );

     /****************************************************************/
     /*                                                              */
     /*                                                              */
     /****************************************************************/
     MQGET(Hcon,                     /* connection handle            */
           Hobj,                     /* object handle                */
           &md,                      /* message descriptor           */
           &gmo,                     /* GET options                  */
           messlen,                  /* buffer length                */
           &buffer,                  /* message buffer               */
           &messlen,                 /* message length               */
           &CompCode,                /* completion code              */
           &Reason);                 /* reason code                  */

     /* report reason, if any */
     printf("=== MQGET ended with reason code %ld ===\n", Reason);
     if (Reason != MQRC_NONE)
     {
       CompCode = MQCC_FAILED;
     }
     else
     {
        sOrder_Reply = (void *)buffer;
        printf("############################################\n");
        printf("### ExecOrder: MQSWF Order Process       ###\n");
        printf("###           Reponse received           ###\n");
        printf("############################################\n");
        printf("Response: Order_Message.ReturnCode : %ld\n",
                               sOrder_Reply->ReturnCode );
     }
   }
   if (OpenCode != MQCC_FAILED)
   {
     C_options = 0;                  /* no close options             */
     MQCLOSE(Hcon,                   /* connection handle            */
             &Hobj,                  /* object handle                */
             C_options,
             &CompCode,              /* completion code              */
             &Reason);               /* reason code                  */

     /* report reason, if any     */
     if (Reason != MQRC_NONE)
     {
       printf("Error: MQCLOSE ended with reason code %ld\n", Reason);
     }
   }

   /******************************************************************/
   /*                                                                */
   /*   Disconnect from MQM if not already connected                 */
   /*                                                                */
   /******************************************************************/
   if (CReason != MQRC_ALREADY_CONNECTED)
   {
     MQDISC(&Hcon,                   /* connection handle            */
            &CompCode,               /* completion code              */
            &Reason);                /* reason code                  */

     /* report reason, if any     */
     if (Reason != MQRC_NONE)
     {
       printf("Error: MQDISC ended with reason code %ld\n", Reason);
     }
   }

   /******************************************************************/
   /*                                                                */
   /* END OF ExecOrder                                               */
   /*                                                                */
   /******************************************************************/
   printf("### sample ExecOrder end\n");
   return(0);
 }
